from qgis.core import *
import math
import random

# Definiere eine benutzerdefinierte Funktion für QGIS
@qgsfunction(args='auto', group='Custom')

def Zufallspunkte_hoch(xInterval, yInterval, rand, position_seed, filename_seed, buffer_distance, scale, feature, parent):

    # Initialisiere die Zufallszahlengenerierung mit einem Saatwert für reproduzierbare Ergebnisse
    random.seed(position_seed)

    # Berechne die Begrenzungsbox des aktuellen Features
    box = feature.geometry().boundingBox()

    # Berechne, wie viele Punkte basierend auf den Intervallen in x- und y-Richtung passen
    countX = math.ceil(box.width() / xInterval)
    countY = math.ceil(box.height() / yInterval)

    # Berechne die neue Größe des Gitters, um die Begrenzungsbox abzudecken
    gridX = countX * xInterval
    gridY = countY * yInterval
    dX = gridX - box.width()
    dY = gridY - box.height()
    xMin = box.xMinimum() - (dX / 2)
    yMin = box.yMinimum() - (dY / 2)

    # Runde den übergebenen Maßstab auf die nächste ganze Zahl
    massstab_attribute = round(float(scale))

    # Überprüfe, ob in der Ebene ‚Gebirgspunkte‘ bereits Punkte mit demselben Maßstab und derselben Gebirgs-Kategorie vorhanden sind
    layer = QgsProject.instance().mapLayersByName('Gebirgspunkte')[0]
    existing_features = layer.getFeatures(QgsFeatureRequest().setFilterExpression(f'Massstab = {massstab_attribute} AND Kategorie = \'hoch\''))

    # Nur fortfahren, wenn keine entsprechenden Punkte vorhanden sind und der Maßstab glatt teilbar ist
    if len(list(existing_features)) > 0 or massstab_attribute % 250000 != 0:
        return None

    # Generiere Punkte innerhalb der Begrenzungsbox basierend auf den angegebenen Intervallen
    points = []
    for xOff in range(countX + 1):
        for yOff in range(countY + 1):
            ptX = xMin + xOff * xInterval + rand * random.uniform(0, xInterval)
            ptY = yMin + yOff * yInterval + rand * random.uniform(0, yInterval)
            pt = QgsPointXY(ptX, ptY)
            points.append(pt)

    # Behalte nur Punkte, die innerhalb der Geometrie des Gebirgs-Polygons liegen
    points = [pt for pt in points if feature.geometry().contains(QgsGeometry.fromPointXY(pt))]

    # Überprüfe und entferne Punkte, die sich innerhalb der Puffer anderer relevanter Ebenen befinden
    for other_layer_name in ['Flüsse 50 Ausschnitt', 'Seen 50', 'Küstenlinie 50 Ausschnitt']:
        other_layer = QgsProject.instance().mapLayersByName(other_layer_name)[0]
        other_features = other_layer.getFeatures()

        for other_feature in other_features:
            other_geometry = other_feature.geometry().simplify(0.05)  # Vereinfache die Geometrie zur Optimierung
            buffer_geometry = other_geometry.buffer(buffer_distance, 3)  # Erstelle einen Puffer um die Geometrie
            points = [pt for pt in points if not buffer_geometry.contains(QgsGeometry.fromPointXY(pt))]

    # Sortiere die Punkte nach ihrer y-Position, um einen Tiefeneffekt zu erzielen
    points.sort(key=lambda p: p.y(), reverse=True)

    # Weise jedem Punkt ein zufälliges SVG-Symbol zu
    svg_attribute = []
    random.seed(filename_seed)
    for pt in points:
        svg_number = str(random.randint(22, 63)).zfill(2)
        svg_attribute.append(f'MH{svg_number}.svg')

    # Weise jedem Punkt einen zufälligen Skalierungsfaktor zu
    skalierungsfaktoren = []
    for _ in points:
        # Erzeuge einen zufälligen Skalierungsfaktor in Zehntelschritten von 0.9 bis 1.1
        faktor = round(random.uniform(0.9, 1.1), 1)
        skalierungsfaktoren.append(faktor)

    # Füge die generierten Punkte als neue Features zur Ebene hinzu
    layer.startEditing()
    for svg_name, skalierungsfaktor, point in zip(svg_attribute, skalierungsfaktoren, points):
        new_feature = QgsFeature(layer.fields())
        new_feature.setAttribute('SVG', svg_name)
        new_feature.setAttribute('Massstab', massstab_attribute)
        new_feature.setAttribute('Kategorie', 'hoch')
        new_feature.setAttribute('Skalierung', skalierungsfaktor)  # Setze den Skalierungsfaktor
        new_feature.setGeometry(QgsGeometry.fromPointXY(point))
        layer.addFeature(new_feature)

    # Identifiziere Punkte für die Versatzprüfung basierend auf einem erweiterten Puffer
    vorsortierte_punkte = []
    for other_layer_name in ['Flüsse 50 Ausschnitt', 'Seen 50', 'Küstenlinie 50 Ausschnitt']:
        other_layer = QgsProject.instance().mapLayersByName(other_layer_name)[0]
        other_features = other_layer.getFeatures()

        for other_feature in other_features:
            erweiterter_puffer = other_feature.geometry().buffer(1.5 * buffer_distance, 3)
            for pt in points:
                if erweiterter_puffer.contains(QgsGeometry.fromPointXY(pt)):
                    vorsortierte_punkte.append(pt)

    # Identifiziere ‚eingeklemmte‘ Punkte
    punkte_ohne_versatz = set()
    for pt in vorsortierte_punkte:
        verschobener_pt_rechts = QgsPointXY(pt.x() + buffer_distance / 2, pt.y())
        verschobener_pt_links = QgsPointXY(pt.x() - buffer_distance / 2, pt.y())
        im_puffer_rechts = False
        im_puffer_links = False

        for other_layer_name in ['Flüsse 50 Ausschnitt', 'Seen 50', 'Küstenlinie 50 Ausschnitt']:
            other_layer = QgsProject.instance().mapLayersByName(other_layer_name)[0]
            other_features = other_layer.getFeatures()
            for other_feature in other_features:
                original_puffer = other_feature.geometry().buffer(buffer_distance, 3)
                if original_puffer.contains(QgsGeometry.fromPointXY(verschobener_pt_rechts)):
                    im_puffer_rechts = True
                if original_puffer.contains(QgsGeometry.fromPointXY(verschobener_pt_links)):
                    im_puffer_links = True

        if im_puffer_rechts and im_puffer_links:
            punkte_ohne_versatz.add(pt)

    # Setze Skalierung auf 0.5 für eingeklemmte Punkte, ohne Versatz zu ändern
    for pt in punkte_ohne_versatz:
        for feature in layer.getFeatures():
            if feature.geometry().asPoint() == pt:
                feature.setAttribute('Skalierung', 0.5)
                layer.updateFeature(feature)

    # Prüfe vorsortierte Punkte auf Versatz nach rechts und setze entsprechende Attribute
    for pt in vorsortierte_punkte:
        if pt in punkte_ohne_versatz:
            continue  # Überspringe Punkte ohne Versatz
        verschobener_pt = QgsPointXY(pt.x() + buffer_distance / 2, pt.y())

        for other_layer_name in ['Flüsse 50 Ausschnitt', 'Seen 50', 'Küstenlinie 50 Ausschnitt']:
            other_layer = QgsProject.instance().mapLayersByName(other_layer_name)[0]
            other_features = other_layer.getFeatures()

            for other_feature in other_features:
                original_puffer = other_feature.geometry().buffer(buffer_distance, 3)
                if original_puffer.contains(QgsGeometry.fromPointXY(verschobener_pt)):
                    for feature in layer.getFeatures():
                        if feature.geometry().asPoint() == pt and feature not in punkte_ohne_versatz:
                            feature.setAttribute('Versatz', -1)
                            feature.setAttribute('Skalierung', 0.8)
                            layer.updateFeature(feature)
                            break

    # Prüfe vorsortierte Punkte auf Versatz nach links und setze entsprechende Attribute
    for pt in vorsortierte_punkte:
        if pt in punkte_ohne_versatz:
            continue  # Überspringe Punkte ohne Versatz
        verschobener_pt_links = QgsPointXY(pt.x() - buffer_distance / 2, pt.y())

        for other_layer_name in ['Flüsse 50 Ausschnitt', 'Seen 50', 'Küstenlinie 50 Ausschnitt']:
            other_layer = QgsProject.instance().mapLayersByName(other_layer_name)[0]
            other_features = other_layer.getFeatures()

            for other_feature in other_features:
                original_puffer = other_feature.geometry().buffer(buffer_distance, 3)
                if original_puffer.contains(QgsGeometry.fromPointXY(verschobener_pt_links)):
                    for feature in layer.getFeatures():
                        if feature.geometry().asPoint() == pt and feature not in punkte_ohne_versatz:
                            feature.setAttribute('Versatz', 1)
                            feature.setAttribute('Skalierung', 0.8)
                            layer.updateFeature(feature)
                            break

    # Schreibe Änderungen in Ebene
    layer.commitChanges()

    return QgsGeometry.fromMultiPointXY(points)
