# Dieses Skript zerlegt die Datei „Maulwurfshügel.svg“ in die einzelnen Dateien „MH01.svg“ bis „MH63.svg“. Voraussetzung ist allerdings, dass sich im Hintergrund der 63 Hügel unverändert die 63 unsichtbaren Kreise mit den Pfadnummern 1–63 befinden!
# This script splits the file “Maulwurfshügel.svg” into the individual files “MH01.svg” to “MH63.svg”. However, the prerequisite is that the 63 invisible circles with the path numbers 1-63 remain unchanged in the background of the 63 hills!

from xml.etree import ElementTree as ET
import re
import os
import shutil

def find_existing_svgs(base_dir):
    """
    Findet existierende MH##.svg Dateien im angegebenen Verzeichnis.
    """
    existing_svgs = [f for f in os.listdir(base_dir) if os.path.isfile(os.path.join(base_dir, f)) and f.startswith("MH") and f.endswith(".svg") and f[2:4].isdigit()]
    return existing_svgs

def create_new_alt_directory(base_dir):
    """
    Erstellt ein neues alt##-Verzeichnis im angegebenen Basisverzeichnis.
    """
    existing_alts = [d for d in os.listdir(base_dir) if os.path.isdir(os.path.join(base_dir, d)) and d.startswith("alt")]
    highest_num = 0
    for dir_name in existing_alts:
        try:
            num = int(dir_name[3:])
            highest_num = max(highest_num, num)
        except ValueError:
            continue
    new_dir_name = f"alt{highest_num + 1:02}"
    new_dir_path = os.path.join(base_dir, new_dir_name)
    return new_dir_name, new_dir_path

def move_svgs_to_alt_dir(svgs, alt_dir):
    """
    Verschiebt die gegebenen SVG-Dateien in das angegebene Verzeichnis.
    """
    for svg in svgs:
        shutil.move(svg, alt_dir)

# Überprüfe, ob zu sichernde SVG-Dateien existieren
base_dir = "."  # Aktuelles Verzeichnis als Basisverzeichnis
existing_svgs = find_existing_svgs(base_dir)

if existing_svgs:
    # Erstelle ein neues alt##-Verzeichnis, wenn notwendig
    new_dir_name, new_dir_path = create_new_alt_directory(base_dir)
    os.makedirs(new_dir_path, exist_ok=True)
    # Verschiebe existierende MH##.svg Dateien
    move_svgs_to_alt_dir(existing_svgs, new_dir_path)
    print(f"SVG-Dateien nach {new_dir_name} verschoben.")
else:
    print("Keine SVG-Dateien zum Verschieben gefunden.")

# Die SVG-Namespace-Definition
SVG_NAMESPACE = "{http://www.w3.org/2000/svg}"
ET.register_namespace("", "http://www.w3.org/2000/svg")

# Der Dateiname der ursprünglichen SVG-Datei
original_file_name = "Maulwurfshügel.svg"

def extract_paths(svg_content):
    """
    Extrahiert Pfade aus der SVG-Datei und gruppiert sie nach ihren IDs.
    """
    paths = {}
    tree = ET.ElementTree(ET.fromstring(svg_content))
    for path in tree.findall(f'.//{SVG_NAMESPACE}path'):
        path_id = path.get('id')
        if path_id:
            paths[path_id] = path
    return paths, tree

def calculate_bounding_box(path):
    """
    Berechnet die Bounding Box eines Pfades anhand seiner 'd'-Attribute.
    """
    d_attr = path.get('d')
    coords = list(map(float, re.findall(r"-?\d+\.?\d*", d_attr)))
    x_coords = coords[::2]
    y_coords = coords[1::2]
    return min(x_coords), max(x_coords), min(y_coords), max(y_coords)

def path_in_bounding_box(path, bbox):
    """
    Vereinfachte Überprüfung, ob ein Knotenpunkt eines Pfades innerhalb einer gegebenen Bounding Box liegt.
    """
    d_attr = path.get('d')
    # Extrahiere den ersten Koordinatenpunkt
    coords = list(map(float, re.findall(r"-?\d+\.?\d*", d_attr)[:2]))
    x, y = coords[0], coords[1]
    return bbox[0] <= x <= bbox[1] and bbox[2] <= y <= bbox[3]

def create_svg_with_custom_viewbox(paths, bbox, file_name):
    """
    Erstellt eine neue SVG-Datei mit den gegebenen Pfaden und einer angepassten viewBox.
    """
    viewBox_value = f"{bbox[0]} {bbox[2]} {bbox[1]-bbox[0]} {bbox[3]-bbox[2]}"
    svg_root = ET.Element('svg', attrib={
        'xmlns:svg': "http://www.w3.org/2000/svg",
        'version': "1.1",
        'width': "12mm",
        'height': "12mm",
        'viewBox': viewBox_value
    })
    for path in paths:
        svg_root.append(path)
    tree = ET.ElementTree(svg_root)
    tree.write(file_name, encoding="utf-8", xml_declaration=True)

with open(original_file_name, 'r') as file:
    svg_content = file.read()

paths, original_tree = extract_paths(svg_content)

for path_id in range(1, 64):
    main_path_id = f'path{path_id}'
    main_path = paths.get(main_path_id)
    if main_path is None:
        continue
    bbox = calculate_bounding_box(main_path)
    # Überprüfung für die Zugehörigkeit basiert nun darauf, ob ein Punkt innerhalb der Bounding Box liegt
    included_paths = [main_path] + [path for pid, path in paths.items() if pid != main_path_id and path_in_bounding_box(path, bbox)]

    new_file_name = f"MH{path_id:02}.svg"
    create_svg_with_custom_viewbox(included_paths, bbox, new_file_name)

print("SVG-Dateien wurden erstellt.")
